<?php
namespace EMS\PortalFuncionario\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\{ Request, Response };
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Services\ApuracaoService;

class ApuracaoController {

    /** @var ApuracaoService */
    protected $apuracaoService;

    public function __construct(EntityManager $entityManager, ApuracaoService $apuracaoService) {
        $this->entityManager    = $entityManager;
        $this->apuracaoService  = $apuracaoService;
    }

    public function getPeriodoApuracao(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $params = $request->getParameters();
            $periodoAcompanhamento = $this->apuracaoService->getPeriodoApuracao(
                $params['NRVINCULOM'],
                $params['NRORG']
            );
            $response->addDataSet(new DataSet('periodoApuracao', $periodoAcompanhamento));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getApuracaoDia(DTO\Request\Filter $request, DTO\Response $response) {
        $row = $request->getParameters();

        $nrorg = $row["NRORG"]; 
        $nrvinculom = $row["NRVINCULOM"];
        $nrperiodoapuracao = empty($row["NRPERIODOAPURACAO"]) ? null : $row["NRPERIODOAPURACAO"];

        $apuracaoPorData = $this->apuracaoService->getApuracaoPorData(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao
        );
        $response->addDataSet(new DataSet('apuracaoDia', $apuracaoPorData));
    }

    public function getSaldosPeriodoApuracao(DTO\Request\Filter $request, DTO\Response $response) {
        $row = $request->getParameters();

        $nrorg = $row["NRORG"];
        $nrvinculom = $row["NRVINCULOM"];
        $nrperiodoapuracao = empty($row["NRPERIODOAPURACAO"]) ? null : $row["NRPERIODOAPURACAO"];

        $apuracaoPorData = $this->apuracaoService->getSaldosPeriodoApuracao(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao
        );
        $response->addDataSet(new DataSet('apuracaoDia', $apuracaoPorData));
    }

    public function getSaldoPeriodo(DTO\Request\Filter $request, DTO\Response $response) {
        $row = $request->getParameters();

        $nrorg = $row["NRORG"];
        $nrvinculom = $row["NRVINCULOM"];

        $saldosPeriodoApuracao = $this->apuracaoService->getSaldosPeriodoApuracao(
            $nrvinculom,
            $nrorg
        );
        $response->addDataSet(new DataSet('saldoPeriodo', $saldosPeriodoApuracao));
    }

    public function getResumoSaldoBanco(DTO\Request\Filter $request, DTO\Response $response) {
        $row = $request->getParameters();

        $nrorg = $row["NRORG"];
        $nrvinculom = $row["NRVINCULOM"];

        $nrperiodoapuracao = empty($row["NRPERIODOAPURACAO"]) ? null : $row["NRPERIODOAPURACAO"];

        $saldosPeriodoApuracao = $this->apuracaoService->getResumoSaldoBanco(
            $nrvinculom,
            $nrorg,
            $nrperiodoapuracao
        );

        $response->addDataSet(new DataSet('data', $saldosPeriodoApuracao));
    }

    public function apurarDias(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
    
            $nrorg              = $row['NRORG'];
            $nrvinculom         = $row['NRVINCULOM'];
            $dias               = $row['DIAS'];
    
            $apuracaoBemSucedida = $this->apuracaoService->apurarDias(
                $nrorg,
                $nrvinculom,
                $dias
            );
    
            $response->addDataSet(new DataSet('data', $apuracaoBemSucedida));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function apurarPeriodo(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $nrorg              = $row['NRORG'];
        $nrvinculom         = $row['NRVINCULOM'];
        $nrperiodoapuracao  = $row['NRPERIODOAPURACAO'];

        $apuracaoBemSucedida = $this->apuracaoService->apurarPeriodo(
            $nrorg,
            $nrvinculom,
            $nrperiodoapuracao
        );

        $response->addDataSet(new DataSet('data', $apuracaoBemSucedida));
    }

    public function verificaDiaTemResumo(
        Request\Filter $request,
        Response       $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int)    $row['NRORG'];
            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $temResumo = $this->apuracaoService->verificaDiaTemResumo(
                $nrOrg,
                $nrVinculo,
                $date
            );

            $dataset = [
                'temResumo' => $temResumo
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function apurarPeriodoFuncionario(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg     = (int) $row['NRORG'];
            $nrVinculo = (int) $row['NRVINCULOM'];
            $nrPeriodo = (int) $row['NRPERIODOAPURACAO'];

            $success = $this->apuracaoService->apurarPeriodoFuncionario(
                $nrOrg,
                $nrVinculo,
                $nrPeriodo
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
