/*
Descrio:
Funo responsvel por retornar o cdigo reduzido das contas contabeis
referentes a movimentao de estoque, para exportao de dados para o
sistema MX  Manager.

Rotina especifica do cliente Hospital Aliana

Criao:
12/06/2012 - Jefferson de Melo Gouvea

@C:\web_575\sql\end_conv\oracle\clientes\hal\0006_HALFUNO.SQL;

SELECT * FROM TABLE(HAL_CODREDUZ(NULL, NULL, NULL, NULL)
/

Parmetros:
P_CDFILIAL   	-> Cdigo da filial
P_CDARVPROD    -> Cdigo do produto para exibio em rvore
P_TIPOMOVI		-> Tipo de movimentao:
							0 - Implantao no estoque
                     1 - Ajuste no estoque
                     2 - Entrada no estoque
                     3 - Retirada no estoque/Retirada Planejamento
                     4 - Saida no estoque
                     8 - Retirada (Vendas FORSALE - custo do produto)
                     9 - Retirada (Vendas FORSALE - venda)
P_CDTIPORETI	->	Cdigo do tipo de retirada
*/

DROP TYPE HAL_CODREDUZ_TABLE
/

DROP TYPE HAL_CODREDUZ_TYPE
/

CREATE TYPE HAL_CODREDUZ_TYPE AS
OBJECT (IDTIPOLAN		VARCHAR2(01),
        CDREDUCONT	VARCHAR2(20),
		  CDHISTPADR	VARCHAR2(04),
		  DSHISTPADR	VARCHAR2(60));
/

CREATE TYPE HAL_CODREDUZ_TABLE AS
TABLE OF HAL_CODREDUZ_TYPE
/

CREATE OR REPLACE FUNCTION HAL_CODREDUZ
(P_CDFILIAL      IN FILIAL.CDFILIAL%TYPE,
 P_CDALMOXARIFE  IN ALMOXARIFADO.CDALMOXARIFE%TYPE,
 P_TIPOMOVI      IN PARAMESTOQUE.CDPARESTOQ%TYPE,
 P_VRAJUSPOS     IN FLOAT,
 P_VRAJUSNEG     IN FLOAT,
 P_CDALMOXSAID   IN ALMOXARIFADO.CDALMOXARIFE%TYPE
) RETURN HAL_CODREDUZ_TABLE PIPELINED
IS
   /*Variavel de Debito*/
   V_CDREDUCONTD 				VARCHAR2(20);
   /*Variavel de Credito*/
   V_CDREDUCONTC 				VARCHAR2(20);

   V_CDHISTPADR				VARCHAR2(04);
   V_DSHISTPADR				VARCHAR2(60);
   OUT_RECORD HAL_CODREDUZ_TYPE := HAL_CODREDUZ_TYPE(NULL, NULL, NULL, NULL);
BEGIN
   V_CDREDUCONTC := NULL;
   V_CDREDUCONTD := NULL;
   V_CDHISTPADR  := NULL;
   V_DSHISTPADR  := NULL;

   /*Implantao de estoque*/
   IF P_TIPOMOVI = 0 THEN

      IF P_CDALMOXARIFE = '100' THEN
         V_CDREDUCONTC := '5121010005';
         V_CDREDUCONTD := '1131010003';
      ELSE
         IF P_CDALMOXARIFE = '101' THEN
            V_CDREDUCONTC := '5121010005';
            V_CDREDUCONTD := '1131010006';
         ELSE
            IF P_CDALMOXARIFE = '102' THEN
               V_CDREDUCONTC := '5121010005';
               V_CDREDUCONTD := '1131010004';
            END IF;
        END IF;
      END IF;

      /*Busca historico padrao*/
      SELECT MAX(P.CDHISTPADRAJIMP), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRAJIMP;
   END IF;

   /*Ajuste de inventrio */
   IF P_TIPOMOVI = 1 THEN
      /* positivo */
      IF P_VRAJUSPOS > 0 THEN
         IF P_CDALMOXARIFE = '100' THEN
            V_CDREDUCONTC := '5121010005';
            V_CDREDUCONTD := '1131010003';
         ELSE
            IF P_CDALMOXARIFE = '101' THEN
               V_CDREDUCONTC := '5121010005';
               V_CDREDUCONTD := '1131010006';
            ELSE
               IF P_CDALMOXARIFE = '102' THEN
                  V_CDREDUCONTC := '5121010005';
                  V_CDREDUCONTD := '1131010004';
               END IF;
           END IF;
         END IF;
      END IF;

      /* negativo */
      IF P_VRAJUSNEG > 0 THEN
         IF P_CDALMOXARIFE = '100' THEN
            V_CDREDUCONTC := '1131010003';
            V_CDREDUCONTD := '5121010005';
         ELSE
            IF P_CDALMOXARIFE = '101' THEN
               V_CDREDUCONTC := '1131010006';
               V_CDREDUCONTD := '5121010005';
            ELSE
               IF P_CDALMOXARIFE = '102' THEN
                  V_CDREDUCONTC := '1131010004';
                  V_CDREDUCONTD := '5121010005';
               END IF;
           END IF;
         END IF;
      END IF;

      /*Busca historico padrao*/
      SELECT MAX(P.CDHISTPADRAJIMP), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRAJIMP;
   END IF;

   /*Entradas no estoque*/
   IF P_TIPOMOVI = 2 THEN

      IF P_CDALMOXSAID IS NULL THEN
         IF P_CDALMOXARIFE = '100' THEN
            V_CDREDUCONTC := '5121050001';
            V_CDREDUCONTD := '1131010003';
         ELSE
            IF P_CDALMOXARIFE = '101' THEN
               V_CDREDUCONTC := '5121050001';
               V_CDREDUCONTD := '1131010006';
            ELSE
               IF P_CDALMOXARIFE = '102' THEN
                  V_CDREDUCONTC := '5121050001';
                  V_CDREDUCONTD := '1131010004';
               END IF;
           END IF;
         END IF;
      ELSE
         IF P_CDALMOXARIFE = '100' THEN
            IF P_CDALMOXSAID = '101' THEN
               V_CDREDUCONTC := '1131010003';
               V_CDREDUCONTD := '1131010006';
            ELSE
               IF P_CDALMOXSAID = '102' THEN
                  V_CDREDUCONTC := '1131010003';
                  V_CDREDUCONTD := '1131010004';
               END IF;
            END IF;
         ELSE
            IF P_CDALMOXARIFE = '101' THEN
               IF P_CDALMOXSAID = '100' THEN
                  V_CDREDUCONTC := '1131010006';
                  V_CDREDUCONTD := '1131010003';
               ELSE
                  IF P_CDALMOXSAID = '102' THEN
                     V_CDREDUCONTC := '1131010006';
                     V_CDREDUCONTD := '1131010004';
                  END IF;
               END IF;
            ELSE
               IF P_CDALMOXARIFE = '102' THEN
                  IF P_CDALMOXSAID = '100' THEN
                     V_CDREDUCONTC := '1131010004';
                     V_CDREDUCONTD := '1131010003';
                  ELSE
                     IF P_CDALMOXSAID = '101' THEN
                        V_CDREDUCONTC := '1131010004';
                        V_CDREDUCONTD := '1131010006';
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;

      /*Busca historico padrao*/
      SELECT MAX(A.CDHISTPADR), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM TIPOOPERAC A, HISTPADR H
       WHERE A.CDTIPOOPER   = '10'
         AND A.IDENTRSAIDOP = 'E'
         AND H.CDHISTPADR   = A.CDHISTPADR;
   END IF;

   /* Retiradas de estoque/Retirada Planejamento */
   IF P_TIPOMOVI = 3 THEN
      IF P_CDALMOXARIFE = '100' THEN
         V_CDREDUCONTC := '1131010003';
         V_CDREDUCONTD := '5121050001';
      ELSE
         IF P_CDALMOXARIFE = '101' THEN
            V_CDREDUCONTC := '1131010006';
            V_CDREDUCONTD := '5121050001';
         ELSE
            IF P_CDALMOXARIFE = '102' THEN
               V_CDREDUCONTC := '1131010004';
               V_CDREDUCONTD := '5121050001';
            END IF;
        END IF;
      END IF;

      /*Busca historico padrao*/
      SELECT MAX(P.CDHISTPADRETEST), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRETEST;
   END IF;

   /*Saidas de estoque*/
   IF P_TIPOMOVI = 4 THEN
      IF P_CDALMOXARIFE = '100' THEN
         V_CDREDUCONTC := '1131010003';
         V_CDREDUCONTD := '5121050001';
      ELSE
         IF P_CDALMOXARIFE = '101' THEN
            V_CDREDUCONTC := '1131010006';
            V_CDREDUCONTD := '5121050001';
         ELSE
            IF P_CDALMOXARIFE = '102' THEN
               V_CDREDUCONTC := '1131010004';
               V_CDREDUCONTD := '5121050001';
            END IF;
        END IF;
      END IF;

      SELECT MAX(P.CDHISTPADRETEST), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRETEST;
   END IF;

   /*Retiradas de estoque (Vendas FORSALE - custo do produto) */
   IF P_TIPOMOVI = 8 THEN
      IF P_CDALMOXARIFE = '100' THEN
         V_CDREDUCONTC := '1131010003';
         V_CDREDUCONTD := '5121050001';
      ELSE
         IF P_CDALMOXARIFE = '101' THEN
            V_CDREDUCONTC := '1131010006';
            V_CDREDUCONTD := '5121050001';
         ELSE
            IF P_CDALMOXARIFE = '102' THEN
               V_CDREDUCONTC := '1131010004';
               V_CDREDUCONTD := '5121050001';
            END IF;
        END IF;
      END IF;

      /*Busca historico padrao*/
      SELECT MAX(P.CDHISTPADRETEST), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRETEST;
   END IF;

   /*Retiradas de estoque (Vendas FORSALE - venda) */
   IF P_TIPOMOVI = 9 THEN
      V_CDREDUCONTC := '4112010001';
      V_CDREDUCONTD := '1125060002';

      /*Busca historico padrao*/
      SELECT MAX(P.CDHISTPADRETEST), MAX(H.DSHISTPADR)
        INTO V_CDHISTPADR, V_DSHISTPADR
        FROM PARAMESTOQUE P, HISTPADR H
       WHERE P.CDPARESTOQ = '1'
         AND H.CDHISTPADR = P.CDHISTPADRETEST;
   END IF;

   /*Credito*/
   OUT_RECORD.IDTIPOLAN  := 'C';
   OUT_RECORD.CDREDUCONT := V_CDREDUCONTC;
   OUT_RECORD.CDHISTPADR := TRUNC(V_CDHISTPADR);
   OUT_RECORD.DSHISTPADR := V_DSHISTPADR;
   PIPE ROW(OUT_RECORD);
   /*Debito*/
   OUT_RECORD.IDTIPOLAN  := 'D';
   OUT_RECORD.CDREDUCONT := V_CDREDUCONTD;
   OUT_RECORD.CDHISTPADR := TRUNC(V_CDHISTPADR);
   OUT_RECORD.DSHISTPADR := V_DSHISTPADR;
   PIPE ROW(OUT_RECORD);
END;
/

/*Otimizao*/
CREATE INDEX HAL_INTG_CTBL_EST ON LANCTOESTOQ (CDFILIMOVI ASC, IDTPLANCTO ASC, DTLANCESTQ ASC)
/

-- VERSAO --
BEGIN ATUVEROBJ('HAL_CODREDUZ','5.75.XXX'); END;
/
